/*
 * init.c
 */

#include "model.h"

extern Firm firm[NFirm],obs[NFirm][Time];
extern Worker worker[NWorker];
extern Consumer consumer[NConsumer];
extern Holder holder[NHolder];
extern Env env;
extern long seed;

void ini_agent(){
	int i,j,t,k,flag;
	double tmp,tmpi,nf,nw,nh;

	nf = (double)(NFirm);
	nw = (double)(NWorker);
	nh = (double)(NHolder);

	env.upcsr = 0.1;	// s_1
	env.depr = 0.1;		// s_3
	env.beta = 0.1;		// s_2
	env.iner = 1.0;		// s_5
	env.eff = 0.0;		// s_4
	env.scsr = 0.2;		// s_6 used only in cutScr function
	env.abl = 1.0;		// A
	env.ablr = 0.1;		// A
	env.iunit = 1.0;	// investment unit

	tmp = nw / nf * (env.eff + env.abl) + nh / nf / env.depr * env.beta;
	for(i=0;i<NFirm;i++){
		tmpi = tmp * (1.0 - env.upcsr * firm[i].csr);
		firm[i].csr = (double)(i) / nf;
		firm[i].quantity = tmpi;
		firm[i].wage = tmpi / nw * nf;
		firm[i].demand = tmpi;
		firm[i].invest = nh / nf;
		firm[i].stock = nh / nf / env.depr;
	}
	for(i=0;i<NWorker;i++){
		worker[i].abl = (ran1(&seed) * 2.0 - 1.0) * env.ablr + env.abl;
		worker[i].csr = ran1(&seed);
//		worker[i].csr = 0.0;
		for(j=0;j<Memory;j++){
			t = (int)(ran1(&seed) * NFirm);
			flag = 0;
			for(k=0;k<j;k++)if(worker[i].list.firm[k] == t) flag = 1;
			if(flag==0) worker[i].list.firm[j] = t;
			else j--;
		}
	}
	for(i=0;i<NConsumer;i++){
		consumer[i].csr = ran1(&seed);
//		consumer[i].csr = 0.0;
		for(j=0;j<Memory;j++){
			t = (int)(ran1(&seed) * NFirm);
			flag = 0;
			for(k=0;k<j;k++)if(consumer[i].list.firm[k] == t) flag = 1;
			if(flag==0) consumer[i].list.firm[j] = t;
			else j--;
		}
	}
	for(i=0;i<NHolder;i++){
		holder[i].csr = ran1(&seed);
//		holder[i].csr = 0.0;
		for(j=0;j<Memory;j++){
			t = (int)(ran1(&seed) * NFirm);
			flag = 0;
			for(k=0;k<j;k++)if(holder[i].list.firm[k] == t) flag = 1;
			if(flag==0) holder[i].list.firm[j] = t;
			else j--;
		}
	}

}

void initialize(int r){
	seed = -5L * (r+1);
	ran1(&seed);	// Initialsing a random seed
	ini_agent();
}

void ini_ran(){
	int i,t;
	for(i=0;i<NFirm;i++)for(t=0;t<Time;t++){
		for(t=0;t<Time;t++) obs[i][t].quantity = 0.0;
		for(t=0;t<Time;t++) obs[i][t].wage = 0.0;
		for(t=0;t<Time;t++) obs[i][t].demand = 0.0;
		for(t=0;t<Time;t++) obs[i][t].invest = 0.0;
		for(t=0;t<Time;t++) obs[i][t].stock = 0.0;
	}
}

void obsF(int time){
	int i,j,k;
	double tmp;

	for(i=0;i<NFirm;i++){
		tmp = obs[i][time].quantity + firm[i].quantity;
		obs[i][time].quantity = tmp;
		tmp = obs[i][time].wage + firm[i].wage;
		obs[i][time].wage = tmp;
		tmp = obs[i][time].demand + firm[i].demand;
		obs[i][time].demand = tmp;
		tmp = obs[i][time].stock + firm[i].stock;
		obs[i][time].stock = tmp;
		tmp = obs[i][time].invest;
		for(j=0;j<NWorker;j++) for(k=0;k<Memory;k++)
		if(i == worker[j].list.firm[k]) tmp += 1.0;
		for(j=0;j<NConsumer;j++) for(k=0;k<Memory;k++)
		if(i == consumer[j].list.firm[k]) tmp += 1.0;
		for(j=0;j<NHolder;j++) for(k=0;k<Memory;k++)
		if(i == holder[j].list.firm[k]) tmp += 1.0;
		obs[i][time].invest = tmp;
	}
}

void printResF(){
// /*
// Final Status
	int i,t,t1;
	t1 = Time - 1;
	for(i=0;i<NFirm;i++) for(t=0;t<Time;t++) {
		obs[i][t].csr = firm[i].csr;
		(obs[i][t].quantity) /= Random;
		(obs[i][t].wage) /= Random;
		(obs[i][t].demand) /= Random;
		(obs[i][t].invest) /= Random;
		(obs[i][t].stock) /= Random;
	}
	printf("CSR,Quality,Wage,Demand,Stock,List\n");
	for(i=0;i<NFirm;i++){
		printf("%5.3f",obs[i][t1].csr);
		printf(",%5.3f",obs[i][t1].quantity);
		printf(",%5.3f",obs[i][t1].wage);
		printf(",%5.3f",obs[i][t1].demand);
		printf(",%5.3f",obs[i][t1].stock);
		printf(",%5.0f",obs[i][t1].invest);
		printf("\n");
	}
// */
/*
// transitions
	int i,t;
	for(i=0;i<NFirm;i++) for(t=0;t<Time;t++) {
		obs[i][t].csr = firm[i].csr;
		(obs[i][t].quantity) /= Random;
		(obs[i][t].wage) /= Random;
		(obs[i][t].demand) /= Random;
		(obs[i][t].invest) /= Random;
		(obs[i][t].stock) /= Random;
	}
	printf("time");
	for(i=0;i<NFirm;i++) printf(",CSR(%.2f)",firm[i].csr);
	printf("\n");
	for(t=0;t<Time;t++){
		printf("%2d",t);
		for(i=0;i<NFirm;i++) printf(",%5.1f",obs[i][t].stock);
		printf("\n");
	}
*/
}
